class Commons:
  UIInstance = None
  TextfieldInputs = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890/:@#.$=_-[]{}()<>|%!+~? '
  TextfieldInputsAlphaNum = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890'
